(function()
{
  /** custom settings **/

  var MAX_POPUP_WIDTH = 800
    , MAX_POPUP_HEIGHT = 700
    , USE_CUSTOM_SCROLLBAR = true
    , SCRIPTS_BASE_URL = document.currentScript.src.substr(0, document.currentScript.src.lastIndexOf('/')+1)
    , MOBILE_DISPLAY_MAX_WIDTH = 740
    , FILL_AVAILABLE_WIDTH_PERCENT = [ .8, .9 ] // [ desktop, mobile ]
    , FILL_AVAILABLE_HEIGHT_PERCENT = [ .8, .9 ] // [ desktop, mobile ]

  /** end of custom settings - stop editing **/

  var initializeStylesheets = function()
  {
    // add the new banner UI styles
    var style = document.createElement('link')
    style.rel = 'stylesheet'
    style.href = SCRIPTS_BASE_URL + 'style.css?1'
    document.head.appendChild(style)

    if ( USE_CUSTOM_SCROLLBAR && ! ( 'simpleBar' in window ) ) {
      var scrollcss = document.createElement('link')
      scrollcss.rel = 'stylesheet'
      scrollcss.href = SCRIPTS_BASE_URL + 'lib/simplebar/simplebar.min.css'
      document.head.appendChild(scrollcss)

      var scrolljs = document.createElement('script')
      scrolljs.type = 'text/javascript'
      scrolljs.src = SCRIPTS_BASE_URL + 'lib/simplebar/simplebar.min.js'
      ;(document.body||document.head).appendChild(scrolljs)
    }
  }

  // hide default CookieFirst panel
  initializeStylesheets()

  // await CookieFirst helpers to load
  var helper_interval_id = setInterval(function(then)
  {
    if ( 'CookieFirst' in window && 'object' == typeof window.CookieFirst ) {
      clearInterval(helper_interval_id)
      then()
    }
  }, 100, function()
  {
    var container = document.getElementById('cookiefirst-custom-banner')
      , adjustContainerPopup, details_button, initSimpleBar, did_init_simplebar, policy_content_scroll, closePopup
      , is_mobile_display, actions

    is_mobile_display = function()
    {
      return (window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth) <= MOBILE_DISPLAY_MAX_WIDTH
    }

    CookieFirst.openPanel = function()
    {
      if ( this.stealthMode )
        throw new Error('CF:: Can\'t open/hide panel in stealth mode')

      if ( ! container )
        throw new Error('CF:: Could not locate the banner element in the DOM.')

      container.style.display = ''

      // adjust scroll to top
      document.documentElement.scrollTop = 0
      document.body && (document.body.scrollTop = 0)

      // disable scroll to enable sticky popup
      document.documentElement.classList.add('cookiefirst-custom-no-overflow')

      // position popup in the center of the user screen
      adjustContainerPopup(container)
    }

    adjustContainerPopup = function(container)
    {
      container = container || document.getElementById('cookiefirst-custom-banner')

      if ( ! container )
        return

      var inner = container.querySelector('#cf-custom-inner')

      if ( ! inner || ! inner.offsetParent )
        return

      var labels = container.querySelectorAll('#place-preference-tools > label')

      if ( is_mobile_display() ) {
        container.classList.add('cf-custom-mobile')
        for ( var i=0; i<labels.length; i++ ) {
          labels[i].classList.add('w-1/2')
          labels[i].classList.remove('w-1/4')
        }
      } else {
        container.classList.remove('cf-custom-mobile')
        for ( var i=0; i<labels.length; i++ ) {
          labels[i].classList.remove('w-1/2')
          labels[i].classList.add('w-1/4')
        }
      }

      // take up 80% screen width/height max
      var client_width = window.innerWidth || document.documentElement.clientWidth || document.body.clientWidth
        , client_height = window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight
        , available_width = Math.floor(client_width * FILL_AVAILABLE_WIDTH_PERCENT[ is_mobile_display() ? 1 : 0 ])
        , available_height = Math.floor(client_height * FILL_AVAILABLE_HEIGHT_PERCENT[ is_mobile_display() ? 1 : 0 ])
        , max_width = MAX_POPUP_WIDTH
        , max_height = MAX_POPUP_HEIGHT
        , assign_width = Math.min( available_width, max_width )
        , assign_height
        , current_height

      inner.style.width = assign_width + 'px'
      inner.style.left = (client_width - assign_width) /2 + 'px'

      inner.style.height = ''
      current_height = inner.offsetHeight
      assign_height = Math.min( available_height, current_height )

      inner.style.height = assign_height + 'px'
      inner.style.top = (client_height - assign_height) /2 + 'px'
    }

    // listen for window resize events and reposition the popup in case activated
    window.addEventListener('resize', adjustContainerPopup.bind(null, container), false)

    initSimpleBar = function()
    {
      if ( ! USE_CUSTOM_SCROLLBAR || did_init_simplebar )
        return

      // hide default browser scrollbar
      'SimpleBar' in window && (new SimpleBar(document.getElementById('cf-custom-policy-content')), did_init_simplebar=true)

      var content_window = container.querySelector('#cf-custom-policy-content .simplebar-content-wrapper')

      content_window && content_window.addEventListener('scroll', function(e)
      {
        var scrollbar = document.getElementById('cf-custom-scrollbar').children[1]
          , scroll_percent = ( 100*e.target.scrollTop ) / (e.target.scrollHeight - e.target.clientHeight)
          , top = e.target.offsetHeight * (scroll_percent *.01)

        scrollbar.parentElement.style.height = e.target.offsetHeight + 'px'

        scrollbar.style.top = Math.max(Math.min( e.target.offsetHeight - scrollbar.offsetHeight, top ), 0) + 'px'
      }, false)
    }

    details_button = container.querySelector('#cf-custom-toggle-details')

    details_button && details_button.addEventListener('click', function(e)
    {
      e.preventDefault()

      var icon = this.querySelector('svg')
        , content = container.querySelector('#cf-custom-policy-content')
        , scrollbar = document.getElementById('cf-custom-scrollbar')

      if ( this.classList.contains('cf-custom-collapsed') ) { // show details
        this.classList.remove('cf-custom-collapsed')
        icon.style.transform = 'rotate(180deg)'
        content.classList.remove('hidden')
        // scroll to content
        setTimeout(function()
        {
          var scroll = content.offsetTop
            , inner = container.querySelector('#cf-custom-inner')
            , scrollTop_prev
            , scroll_interval = setInterval(function()
          {
            scrollTop_prev = inner.scrollTop
            inner.scrollTop += 10

            scrollbar.style.height = content.offsetHeight + 'px'

            if ( scroll-- <= 0 || (scrollTop_prev == inner.scrollTop && scrollTop_prev > 1) ) { // Safari issue fix
              clearInterval( scroll_interval )
              // update positioning of the popup
              adjustContainerPopup(container)
            }
          }, 10)
        }, 10)

        // show scrollbar
        content.querySelector('.simplebar-wrapper') && (scrollbar.classList.remove('hidden'), scrollbar.classList.add('flex'))
      } else { // hide details
        this.classList.add('cf-custom-collapsed')
        icon.style.transform = ''
        content.classList.add('hidden')
        // hide scrollbar
        scrollbar.classList.add('hidden')
        scrollbar.classList.remove('flex')
        // update positioning of the popup
        adjustContainerPopup(container)
      }

      // initialize simplebar in case not done previously
      initSimpleBar()
    }, false)

    // initialize simplebar in case not done previously
    initSimpleBar()

    closePopup = function(container)
    {
      document.documentElement.classList.remove('cookiefirst-custom-no-overflow')

      container = container || document.getElementById('cookiefirst-custom-banner')

      if ( ! container || ! container.offsetParent )
        return

      container.style.display = 'none'

      details_button = details_button || container.querySelector('#cf-custom-toggle-details')

      // close details after closing popup
      if ( ! details_button.classList.contains('cf-custom-collapsed') )
        details_button.click()
    }

    CookieFirst.closePanel = closePopup.bind(null,null)

    // commented out to prevent closing the popups
    /*// exit events
    document.getElementById('cf-custom-overlay').addEventListener('click', closePopup.bind(null,container), false)

    // on esc keyboard
    window.addEventListener('keydown', function(e)
    {
      if ( e.which == 27 ) {
        closePopup(container)
      }
    }, false)*/

    // actions callbacks
    actions = document.getElementById('cf-custom-ui-actions')

    if ( actions && actions.children.length > 0 ) {
      for ( var i=0; i<actions.children.length; i++ ) {
        actions.children[i].addEventListener('click', function(e)
        {
          e.preventDefault()

          var checkboxes = document.querySelectorAll('#place-preference-tools input[type=checkbox]')
            , new_consent = {}
            , is_initial = ! CookieFirst.consent

          for ( var i=0; i<checkboxes.length; i++ ) {
            checkboxes[i].checked = 'confirm_all' == this.name || checkboxes[i].checked
            new_consent[ checkboxes[i].name ] = checkboxes[i].checked
          }

          CookieFirst.updateConsent(new_consent)
          setTimeout(closePopup.bind(null,null), 100)

          if ( is_initial ) {
            // bugfix: CookieFirst does not reload when setting consent for the first time, instead blocks the UI scroll via CSS body class
            var reload_interval_id = setInterval(function()
            {
              // reload window once the consent update API call is finished
              CookieFirst.consent && (clearInterval(reload_interval_id), location.reload())
            }, 200)
          }
        }, false)
      }
    }

    // initial consent notice
    CookieFirst.consent || CookieFirst.openPanel()
  })
  
  // sync CF consent from DOM event
  window.addEventListener('cf_consent', function(e)
  {
    if ( e.detail ) {
      for ( var key in e.detail ) {
        (document.querySelector('#place-preference-tools input[name="' + key + '"]') || {}).checked = e.detail[key]
      }
    }
  }, false)

  // sync policy HTML from current document to popup
  var cf_policy_page = document.getElementById('cookiefirst-policy-page')

  if ( cf_policy_page ) {
    if ( 'MutationObserver' in window ) {
      var mut_observer = new MutationObserver(function(mutations)
      {
        var policy_page = document.getElementById('cf-custom-policy-content-print')

        if ( ! policy_page )
          return

        policy_page.innerHTML = ''
        policy_page.appendChild(cf_policy_page.cloneNode(true))
      })

      mut_observer.observe(cf_policy_page, {
        subtree: true,
        childList: true,
      })
    } else { // older browsers not supporting MutationObserver, use a hack instead
      setInterval(function()
      {
        var policy_page = document.getElementById('cf-custom-policy-content-print')

        if ( ! policy_page )
          return

        policy_page.innerHTML = ''
        policy_page.appendChild(cf_policy_page.cloneNode(true))
      }, 2000)
    }
  }
})()
